/****************************************************************************
 * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
 *
 * Copyright (c) 2006-2010, Kimio Kuramitsu <kimio at ynu.ac.jp>
 *           (c) 2008-      Konoha Team konohaken@googlegroups.com
 * All rights reserved.
 *
 * You may choose one of the following two licenses when you use konoha.
 * See www.konohaware.org/license.html for further information.
 *
 * (1) GNU Lesser General Public License 3.0 (with KONOHA_UNDER_LGPL3)
 * (2) Konoha Software Foundation License 1.0
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/

#include <konoha.h>

#ifdef KONOHA_ON_WINDOWS
#define _USE_MATH_DEFINES
#endif

#include <math.h>

#ifdef __cplusplus 
extern "C" {
#endif

METHOD Math_fabs(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(fabs(Float_to(double, sfp[1])));
}

METHOD Math_abs(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(fabs(Float_to(double, sfp[1])));
}

METHOD Math_pow(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(pow(Float_to(double, sfp[1]),Float_to(double, sfp[2])));
}

METHOD Math_ldexp(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(ldexp(Float_to(double, sfp[1]), Int_to(int, sfp[2])));
}

METHOD Math_modf(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	double iprt = Float_to(double, sfp[2]);
	RETURNf_(modf(Float_to(double, sfp[1]), &iprt));
}

METHOD Math_frexp(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	int exp = Int_to(int, sfp[2]);
	RETURNf_(frexp(Float_to(double, sfp[1]), &exp));
}

METHOD Math_fmod(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(fmod(Float_to(double, sfp[1]),Float_to(double, sfp[2])));
}

METHOD Math_ceil(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(ceil(Float_to(double, sfp[1])));
}

#if KONOHA_ON_WINDOWS
METHOD Math_round(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(round(Float_to(double, sfp[1])));
}

METHOD Math_nearbyint(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(nearbyint(Float_to(double, sfp[1])));
}
#endif

METHOD Math_floor(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(floor(Float_to(double, sfp[1])));
}

METHOD Math_sqrt(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(sqrt(Float_to(double, sfp[1])));
}

METHOD Math_exp(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(exp(Float_to(double, sfp[1])));
}

METHOD Math_log10(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(log10(Float_to(double, sfp[1])));
}

METHOD Math_log(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(log(Float_to(double, sfp[1])));
}

METHOD Math_sin(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(sin(Float_to(double, sfp[1])));
}

METHOD Math_cos(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(cos(Float_to(double, sfp[1])));
}

METHOD Math_tan(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(tan(Float_to(double, sfp[1])));
}

METHOD Math_asin(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(asin(Float_to(double, sfp[1])));
}

METHOD Math_acos(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(acos(Float_to(double, sfp[1])));
}

METHOD Math_atan(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(atan(Float_to(double, sfp[1])));
}

METHOD Math_atan2(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(atan2(Float_to(double, sfp[1]),Float_to(double, sfp[2])));
}

METHOD Math_sinh(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(sinh(Float_to(double, sfp[1])));
}

METHOD Math_cosh(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(cosh(Float_to(double, sfp[1])));
}

METHOD Math_tanh(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(tanh(Float_to(double, sfp[1])));
}

#if defined(KONOHA_ON_WINDOWS)
METHOD Math_asinh(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(asinh(Float_to(double, sfp[1])));
}

METHOD Math_acosh(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(acosh(Float_to(double, sfp[1])));
}

METHOD Math_atanh(Ctx *ctx, knh_sfp_t *sfp, long rix)
{
	RETURNf_(atanh(Float_to(double, sfp[1])));
}
#endif

static knh_FloatData_t FloatConstData[] = {
	{"Math.E", M_E},
	{"Math.LOG2E",M_LOG2E},
	{"Math.LOG10E",M_LOG10E},
	{"Math.LN2",M_LN2},
	{"Math.LN10",M_LN10},
	{"Math.PI", M_PI},
	{"Math.PI_2",M_PI_2},
	{"Math.PI_4",M_PI_4},
	{"Math.SQRT2",M_SQRT2},
	{NULL, K_FLOAT_ZERO}
};

KNHAPI(int) kcheck(void)
{
	return K_BUILDID;
}

KNHAPI(void) init(Ctx *ctx, const knh_PackageLoaderAPI_t *kapi, char *dname, int isOVERRIDE)
{
	kapi->loadFloatData(ctx, FloatConstData);
}

#ifdef __cplusplus
}
#endif

